[PATCH V3 4/4] PCI: ACPI: Add support for PCI device DMA coherency

Suravee Suthikulpanit Suravee.Suthikulpanit at amd.com
Wed Aug 26 06:54:39 PDT 2015


This patch adds support for setting up PCI device DMA coherency from
ACPI _CCA object that should normally be specified in the DSDT node
of its PCI host bridge.

Signed-off-by: Suravee Suthikulpanit <Suravee.Suthikulpanit at amd.com>
CC: Bjorn Helgaas <bhelgaas at google.com>
CC: Catalin Marinas <catalin.marinas at arm.com>
CC: Rob Herring <robh+dt at kernel.org>
CC: Will Deacon <will.deacon at arm.com>
CC: Rafael J. Wysocki <rjw at rjwysocki.net>
CC: Murali Karicheri <m-karicheri2 at ti.com>
---
 drivers/pci/probe.c | 18 +++++++++++++-----
 1 file changed, 13 insertions(+), 5 deletions(-)

diff --git a/drivers/pci/probe.c b/drivers/pci/probe.c
index 4de6594..7cfd8e2 100644
--- a/drivers/pci/probe.c
+++ b/drivers/pci/probe.c
@@ -1551,17 +1551,25 @@ static void pci_init_capabilities(struct pci_dev *dev)
  * @dev: ptr to pci_dev struct of the PCI device
  *
  * Function to update PCI devices's DMA configuration using the same
  * info from the OF node of host bridge's parent (if any).
  */
 static void pci_dma_configure(struct pci_dev *dev)
 {
 	struct device *bridge = pci_get_host_bridge_device(dev);
 
 	if (IS_ENABLED(CONFIG_OF) && dev->dev.of_node) {
-		if (!bridge->parent)
-			return;
-
-		of_dma_configure(&dev->dev, bridge->parent->of_node);
+		if (bridge->parent)
+			of_dma_configure(&dev->dev,
+					 bridge->parent->of_node);
+	} else if (has_acpi_companion(bridge)) {
+		struct acpi_device *adev = to_acpi_node(bridge->fwnode);
+		int ret = acpi_check_dma_coherency(adev);
+
+		if (ret < 0)
+			WARN(1, FW_BUG "PCI device %s fail to setup DMA.\n",
+			     pci_name(dev));
+		else
+			arch_setup_dma_ops(&dev->dev, 0, 0, NULL, ret);
 	}
 
 	pci_put_host_bridge_device(bridge);
@@ -1551,17 +1551,25 @@ static void pci_init_capabilities(struct pci_dev *dev)
  *
  * Function to update PCI devices's DMA configuration using the same
- * info from the OF node of host bridge's parent (if any).
+ * info from the OF node or ACPI node of host bridge's parent (if any).
  */
 static void pci_dma_configure(struct pci_dev *dev)
-- 
2.1.0




More information about the linux-arm-kernel mailing list